/*
	3DOptions.jsx
	Copyright (c) 2012 Motion Boutique
*/


// This class represents the main class of the script
function ThreeDOptions()
{
	// Variable used to keep track of 'this' reference
	var threeDOptions = this;
	
	// Create an instance of the utils class to use its functions
	var utils = new ThreeDOptionsUtils();	
	
	// Script infos
	this.scriptMinSupportVersion = "11.0";
	this.scriptName = "3DOptions.jsx";	
	this.scriptVersion = "1.0";
	this.scriptTitle = "3D Options";
	this.scriptCopyright = "Copyright (c) 2012 Motion Boutique";
	this.scriptHomepage = "http://www.motionboutique.com";
	this.scriptDescription = {en:"This script allows you to quickly modify Geometry and Material Options of selected 3D layers.", 
							  fr:"Ce script permet de modifier rapidement les Options de Géometrie et de Matériaux des calques 3D sélectionnés."};
	this.scriptAbout = {en:this.scriptName + ", v" + this.scriptVersion + "\\r" + this.scriptCopyright + "\\r" + this.scriptHomepage + "\\r\\r" + utils.loc(this.scriptDescription), 
						fr:this.scriptName + ", v" + this.scriptVersion + "\\r" + this.scriptCopyright + "\\r" + this.scriptHomepage + "\\r\\r" + utils.loc(this.scriptDescription)};		
	this.scriptUsage = {en:	"\u25BA In After Effects CS6 or later, run the script \\r" +
							"\u25BA Select at least one 3D layer\\r" +
							"\u25BA Edit parameters",
						fr:	"\u25BA Dans After Effects CS6 ou supérieur, exécuter le script \\r" +
							"\u25BA Sélectionner au moins un calque 3D\\r" +
							"\u25BA Editer les paramètres"};

	// Errors
	this.requirementErr = {en:"This script requires After Effects CS6 or later.", fr:"Ce script nécessite After Effects CS6 ou supérieur."};	
	this.noLayersErr = {en:"Please select at least one layer.", fr:"Veuillez sélectionner au moins un calque."};
	
	// UI strings and default settings
	this.aboutBtnName = "?";
	this.geometryOptionsGrpName = {en:"\u25CF Geometry Options", fr:"\u25CF Geometry Options"};
	this.bevelStyleStName = {en:"Bevel Style:", fr:"Bevel Style:"};
	this.bevelStyleLstChoices = {en:"['None', 'Angular', 'Concave', 'Convex']", fr:"['None', 'Angular', 'Concave', 'Convex']"};
	this.bevelDepthStName = {en:"Bevel Depth:", fr:"Bevel Depth:"};
	this.holeBevelDepthStName = {en:"Hole Bevel Depth:", fr:"Hole Bevel Depth:"};
	this.extrusionDepthStName = {en:"Extrusion Depth:", fr:"Extrusion Depth:"};
	this.curvatureStName = {en:"Curvature:", fr:"Curvature:"};
	this.segmentsStName = {en:"Segments:", fr:"Segments:"};
	this.materialOptionsGrpName = {en:"\u25CF Material Options", fr:"\u25CF Material Options"};
	this.castsShadowsStName = {en:"Casts Shadows:", fr:"Casts Shadows:"};
	this.lightTransmissionStName = {en:"Light Transmission:", fr:"Light Transmission:"};
	this.acceptsShadowsStName = {en:"Accepts Shadows:", fr:"Accepts Shadows:"};
	this.acceptsLightsStName = {en:"Accepts Lights:", fr:"Accepts Lights:"};
	this.appearsInReflectionsStName = {en:"Appears in Reflections:", fr:"Appears in Reflections:"};
	this.ambientCoefStName = {en:"Ambient:", fr:"Ambient:"};
	this.diffuseCoefStName = {en:"Diffuse:", fr:"Diffuse:"};
	this.specularCoefStName = {en:"Specular Intensity:", fr:"Specular Intensity:"};
	this.shininessCoefStName = {en:"Specular Shininess:", fr:"Specular Shininess:"};
	this.metalCoefStName = {en:"Metal:", fr:"Metal:"};
	this.reflectionCoefStName = {en:"Reflection Intensity:", fr:"Reflection Coefficient:"};
	this.glossinessCoefStName = {en:"Reflection Sharpness:", fr:"Reflection Sharpness:"};
	this.fresnelCoefStName = {en:"Reflection Rolloff:", fr:"Reflection Rolloff:"};
	this.transparencyCoefStName = {en:"Transparency Coefficient:", fr:"Transparency Coefficient:"};
	this.transpRolloffStName = {en:"Transparency Rolloff:", fr:"Transparency Rolloff:"};
	this.indexOfRefractionStName = {en:"Index of Refraction:", fr:"Index of Refraction:"};
	
	// Default values
	this.bevelStyleLstVal = 0; // None
	this.bevelDepthVal = 2;
	this.holeBevelDepthVal = 100;
	this.extrusionDepthVal = 0;
	this.curvatureVal = 0;
	this.segmentsVal = 4;
	this.castsShadowsVal = false;
	this.lightTransmissionVal = 0;
	this.acceptsShadowsVal = true;
	this.acceptsLightsVal = true;
	this.appearsInReflectionsVal = true;	
	this.ambientCoefVal = 100;
	this.diffuseCoefVal = 50;
	this.specularCoefVal = 5;
	this.shininessCoefVal = 5;
	this.metalCoefVal = 100;
	this.reflectionCoefVal = 0;
	this.glossinessCoefVal = 100;
	this.fresnelCoefVal = 0;
	this.transparencyCoefVal = 0;
	this.transpRolloffVal = 0;
	this.indexOfRefractionVal = 1;


	// Creates and displays the script interface
	this.buildUI = function (thisObj)
	{
		// dockable panel or palette
		var pal = (thisObj instanceof Panel) ? thisObj : new Window("palette", this.scriptTitle, undefined, {resizeable:false});
		
		// resource specifications
		var res =
		"group { orientation:'column', alignment:['fill','fill'], alignChildren:['right','top'], \
			gr0: Group { \
				aboutBtn: Button { text:'" + this.aboutBtnName + "', preferredSize:[25,20] } \
			}, \
			gra: Group { orientation:'row', alignment:['fill','fill'], \
				st: StaticText { text:'" + utils.loc(this.geometryOptionsGrpName) + "' }, \
				pnl: Panel { alignment:['fill','center'], preferredSize:[-1,1] } \
			}, \
			gr01: Group { orientation:'row', \
                bevelStyleSt: StaticText { text:'" + utils.loc(this.bevelStyleStName) + "' }, \
                bevelStyleLst: DropDownList { properties:{items:" + utils.loc(this.bevelStyleLstChoices) + "}, preferredSize:[150,20] } \
			}, \
			gr02: Group { orientation:'row', \
				bevelDepthSt: StaticText { text:'" + utils.loc(this.bevelDepthStName) + "' }, \
				bevelDepthEt: EditText { text:'" + this.bevelDepthVal + "', characters:5 }, \
				bevelDepthScl: Scrollbar { minvalue:0, maxvalue:100, value:" + this.bevelDepthVal + ", preferredSize:[150,20] } \
			}, \
			gr03: Group { orientation:'row', \
				holeBevelDepthSt: StaticText { text:'" + utils.loc(this.holeBevelDepthStName) + "' }, \
				holeBevelDepthEt: EditText { text:'" + this.holeBevelDepthVal + "', characters:5 }, \
				holeBevelDepthScl: Scrollbar { minvalue:0, maxvalue:100, value:" + this.holeBevelDepthVal + ", preferredSize:[150,20] } \
			}, \
			gr04: Group { orientation:'row', \
				extrusionDepthSt: StaticText { text:'" + utils.loc(this.extrusionDepthStName) + "' }, \
				extrusionDepthEt: EditText { text:'" + this.extrusionDepthVal + "', characters:5 }, \
				extrusionDepthScl: Scrollbar { minvalue:0, maxvalue:1000, value:" + this.extrusionDepthVal + ", preferredSize:[150,20] } \
			}, \
			sep: Panel { alignment:['fill','center'] }, \
			gr05: Group { orientation:'row', \
				curvatureSt: StaticText { text:'" + utils.loc(this.curvatureStName) + "' }, \
				curvatureEt: EditText { text:'" + this.curvatureVal + "', characters:5 }, \
				curvatureScl: Scrollbar { minvalue:-100, maxvalue:100, value:" + this.curvatureVal + ", preferredSize:[150,20] } \
			}, \
			gr06: Group { orientation:'row', \
				segmentsSt: StaticText { text:'" + utils.loc(this.segmentsStName) + "' }, \
				segmentsEt: EditText { text:'" + this.segmentsVal + "', characters:5 }, \
				segmentsScl: Scrollbar { minvalue:0, maxvalue:256, value:" + this.segmentsVal + ", preferredSize:[150,20] } \
			}, \
			grb: Group { orientation:'row', alignment:['fill','fill'], \
				st: StaticText { text:'\u25CF Material Options' }, \
				pnl: Panel { alignment:['fill','center'], preferredSize:[-1,1] } \
			}, \
			gr2: Group { orientation:'row', \
				castsShadowsSt: StaticText { text:'" + utils.loc(this.castsShadowsStName) + "' }, \
				castsShadowsCb: Checkbox { text:'', value:" + this.castsShadowsVal + " }, \
				fooSt: EditText { visible:false } \
			}, \
			gr3: Group { orientation:'row', \
				lightTransmissionSt: StaticText { text:'" + utils.loc(this.lightTransmissionStName) + "' }, \
				lightTransmissionEt: EditText { text:'" + this.lightTransmissionVal + "', characters:5 }, \
				lightTransmissionScl: Scrollbar { minvalue:0, maxvalue:100, value:" + this.lightTransmissionVal + ", preferredSize:[150,20] } \
			}, \
			gr4: Group { orientation:'row', \
				acceptsShadowsSt: StaticText { text:'" + utils.loc(this.acceptsShadowsStName) + "' }, \
				acceptsShadowsCb: Checkbox { text:'', value:" + this.acceptsShadowsVal + " }, \
				fooSt: EditText { visible:false } \
			}, \
			gr5: Group { orientation:'row', \
				acceptsLightsSt: StaticText { text:'" + utils.loc(this.acceptsLightsStName) + "' }, \
				acceptsLightsCb: Checkbox { text:'', value:" + this.acceptsLightsVal + " }, \
				fooSt: EditText { visible:false } \
			}, \
			gr6: Group { orientation:'row', \
				appearsInReflectionsSt: StaticText { text:'" + utils.loc(this.appearsInReflectionsStName) + "' }, \
				appearsInReflectionsCb: Checkbox { text:'', value:" + this.appearsInReflectionsVal + " }, \
				fooSt: EditText { visible:false } \
			}, \
			gr7: Group { orientation:'row', \
				ambientCoefSt: StaticText { text:'" + utils.loc(this.ambientCoefStName) + "' }, \
				ambientCoefEt: EditText { text:'" + this.ambientCoefVal + "', characters:5 }, \
				ambientCoefScl: Scrollbar { minvalue:0, maxvalue:100, value:" + this.ambientCoefVal + ", preferredSize:[150,20] } \
			}, \
			gr8: Group { orientation:'row', \
				diffuseCoefSt: StaticText { text:'" + utils.loc(this.diffuseCoefStName) + "' }, \
				diffuseCoefEt: EditText { text:'" + this.diffuseCoefVal + "', characters:5 }, \
				diffuseCoefScl: Scrollbar { minvalue:0, maxvalue:100, value:" + this.diffuseCoefVal + ", preferredSize:[150,20] } \
			}, \
			gr9: Group { orientation:'row', \
				specularCoefSt: StaticText { text:'" + utils.loc(this.specularCoefStName) + "' }, \
				specularCoefEt: EditText { text:'" + this.specularCoefVal + "', characters:5 }, \
				specularCoefScl: Scrollbar { minvalue:0, maxvalue:100, value:" + this.specularCoefVal + ", preferredSize:[150,20] } \
			}, \
			gr10: Group { orientation:'row', \
				shininessCoefSt: StaticText { text:'" + utils.loc(this.shininessCoefStName) + "' }, \
				shininessCoefEt: EditText { text:'" + this.shininessCoefVal + "', characters:5 }, \
				shininessCoefScl: Scrollbar { minvalue:0, maxvalue:100, value:" + this.shininessCoefVal + ", preferredSize:[150,20] } \
			}, \
			gr11: Group { orientation:'row', \
				metalCoefSt: StaticText { text:'" + utils.loc(this.metalCoefStName) + "' }, \
				metalCoefEt: EditText { text:'" + this.metalCoefVal + "', characters:5 }, \
				metalCoefScl: Scrollbar { minvalue:0, maxvalue:100, value:" + this.metalCoefVal + ", preferredSize:[150,20] } \
			}, \
			gr12: Group { orientation:'row', \
				reflectionCoefSt: StaticText { text:'" + utils.loc(this.reflectionCoefStName) + "' }, \
				reflectionCoefEt: EditText { text:'" + this.reflectionCoefVal + "', characters:5 }, \
				reflectionCoefScl: Scrollbar { minvalue:0, maxvalue:100, value:" + this.reflectionCoefVal + ", preferredSize:[150,20] } \
			}, \
			gr13: Group { orientation:'row', \
				glossinessCoefSt: StaticText { text:'" + utils.loc(this.glossinessCoefStName) + "' }, \
				glossinessCoefEt: EditText { text:'" + this.glossinessCoefVal + "', characters:5 }, \
				glossinessCoefScl: Scrollbar { minvalue:0, maxvalue:100, value:" + this.glossinessCoefVal + ", preferredSize:[150,20] } \
			}, \
			gr14: Group { orientation:'row', \
				fresnelCoefSt: StaticText { text:'" + utils.loc(this.fresnelCoefStName) + "' }, \
				fresnelCoefEt: EditText { text:'" + this.fresnelCoefVal + "', characters:5 }, \
				fresnelCoefScl: Scrollbar { minvalue:0, maxvalue:100, value:" + this.fresnelCoefVal + ", preferredSize:[150,20] } \
			}, \
			gr15: Group { orientation:'row', \
				transparencyCoefSt: StaticText { text:'" + utils.loc(this.transparencyCoefStName) + "' }, \
				transparencyCoefEt: EditText { text:'" + this.transparencyCoefVal + "', characters:5 }, \
				transparencyCoefScl: Scrollbar { minvalue:0, maxvalue:100, value:" + this.transparencyCoefVal + ", preferredSize:[150,20] } \
			}, \
			gr16: Group { orientation:'row', \
				transpRolloffSt: StaticText { text:'" + utils.loc(this.transpRolloffStName) + "' }, \
				transpRolloffEt: EditText { text:'" + this.transpRolloffVal + "', characters:5 }, \
				transpRolloffScl: Scrollbar { minvalue:0, maxvalue:100, value:" + this.transpRolloffVal + ", preferredSize:[150,20] } \
			}, \
			gr17: Group { orientation:'row', \
				indexOfRefractionSt: StaticText { text:'" + utils.loc(this.indexOfRefractionStName) + "' }, \
				indexOfRefractionEt: EditText { text:'" + this.indexOfRefractionVal + "', characters:5 }, \
				indexOfRefractionScl: Scrollbar { minvalue:100, maxvalue:200, value:" + this.indexOfRefractionVal + ", preferredSize:[150,20] } \
			} \
		}"; 
		pal.gr = pal.add(res);
		
		pal.gr.gr2.fooSt.preferredSize.width = 
		pal.gr.gr4.fooSt.preferredSize.width = 
		pal.gr.gr5.fooSt.preferredSize.width = 
		pal.gr.gr6.fooSt.preferredSize.width = 175 + 4 * ($.os.indexOf("Win") == -1);
		
		pal.gr.gr01.bevelStyleLst.preferredSize.width = 200 + 5 * ($.os.indexOf("Win") == -1);
		pal.gr.gr01.bevelStyleLst.selection = this.bevelStyleLstVal;
		
		// event callbacks
		//pal.onResizing = pal.onResize = function () 
		//{
		//	this.layout.resize();
		//};
				
		pal.gr.gr0.aboutBtn.onClick = function () 
		{ 
			utils.createAboutDlg(threeDOptions.scriptAbout, threeDOptions.scriptUsage); 
		};
		
		pal.gr.gr01.bevelStyleLst.onChange = function () 
		{ 
			threeDOptions.bevelStyleLstVal = this.selection.index + 1;
			threeDOptions.process(threeDOptions.getSelectedLayers(), "ADBE Bevel Styles", threeDOptions.bevelStyleLstVal);			
		};

		pal.gr.gr02.bevelDepthEt.onChange = function () 
		{
			if (isNaN(this.text))
			{
				this.text = threeDOptions.bevelDepthVal;
			}
			else
			{
				this.parent.bevelDepthScl.value = parseFloat(this.text);
				threeDOptions.bevelDepthVal = parseFloat(this.text);				
				threeDOptions.process(threeDOptions.getSelectedLayers(), "ADBE Bevel Depth", threeDOptions.bevelDepthVal);
			}
		};
		
		pal.gr.gr02.bevelDepthScl.onChange = pal.gr.gr02.bevelDepthScl.onChanging = function () 
		{
			this.parent.bevelDepthEt.text = this.value;
			threeDOptions.bevelDepthVal = this.value;
			threeDOptions.process(threeDOptions.getSelectedLayers(), "ADBE Bevel Depth", threeDOptions.bevelDepthVal);
		};

		pal.gr.gr03.holeBevelDepthScl.onChange = pal.gr.gr03.holeBevelDepthScl.onChanging = function () 
		{
			this.parent.holeBevelDepthEt.text = this.value;
			threeDOptions.holeBevelDepthVal = this.value;
			threeDOptions.process(threeDOptions.getSelectedLayers(), "ADBE Hole Bevel Depth", threeDOptions.holeBevelDepthVal);
		};
		
		pal.gr.gr03.holeBevelDepthEt.onChange = function () 
		{
			if (isNaN(this.text))
			{
				this.text = threeDOptions.holeBevelDepthVal;
			}
			else
			{
				this.parent.holeBevelDepthScl.value = parseFloat(this.text);
				threeDOptions.holeBevelDepthVal = parseFloat(this.text);				
				threeDOptions.process(threeDOptions.getSelectedLayers(), "ADBE Hole Bevel Depth", threeDOptions.holeBevelDepthVal);
			}
		};
		
		pal.gr.gr04.extrusionDepthEt.onChange = function () 
		{
			if (isNaN(this.text))
			{
				this.text = threeDOptions.extrusionDepthVal;
			}
			else
			{
				this.parent.extrusionDepthScl.value = parseFloat(this.text);
				threeDOptions.extrusionDepthVal = parseFloat(this.text);				
				threeDOptions.process(threeDOptions.getSelectedLayers(), "ADBE Extrsn Depth", threeDOptions.extrusionDepthVal);
			}
		};
		
		pal.gr.gr04.extrusionDepthScl.onChange = pal.gr.gr04.extrusionDepthScl.onChanging = function () 
		{
			this.parent.extrusionDepthEt.text = this.value;
			threeDOptions.extrusionDepthVal = this.value;
			threeDOptions.process(threeDOptions.getSelectedLayers(), "ADBE Extrsn Depth", threeDOptions.extrusionDepthVal);
		};

		pal.gr.gr05.curvatureEt.onChange = function () 
		{
			if (isNaN(this.text))
			{
				this.text = threeDOptions.curvatureVal;
			}
			else
			{
				this.parent.curvatureScl.value = parseFloat(this.text);
				threeDOptions.curvatureVal = parseFloat(this.text);				
				threeDOptions.process(threeDOptions.getSelectedLayers(), "ADBE Plane Curvature", threeDOptions.curvatureVal);
			}
		};
		
		pal.gr.gr05.curvatureScl.onChange = pal.gr.gr05.curvatureScl.onChanging = function () 
		{
			this.parent.curvatureEt.text = this.value;
			threeDOptions.curvatureVal = this.value;
			threeDOptions.process(threeDOptions.getSelectedLayers(), "ADBE Plane Curvature", threeDOptions.curvatureVal);
		};

		pal.gr.gr06.segmentsEt.onChange = function () 
		{
			if (isNaN(this.text))
			{
				this.text = threeDOptions.segmentsVal;
			}
			else
			{
				this.parent.segmentsScl.value = parseFloat(this.text);
				threeDOptions.segmentsVal = parseFloat(this.text);				
				threeDOptions.process(threeDOptions.getSelectedLayers(), "ADBE Plane Subdivision", threeDOptions.segmentsVal);
			}
		};
		
		pal.gr.gr06.segmentsScl.onChange = pal.gr.gr06.segmentsScl.onChanging = function () 
		{
			this.parent.segmentsEt.text = this.value;
			threeDOptions.segmentsVal = this.value;
			threeDOptions.process(threeDOptions.getSelectedLayers(), "ADBE Plane Subdivision", threeDOptions.segmentsVal);
		};
		
		pal.gr.gr2.castsShadowsCb.onClick = function ()
		{
			threeDOptions.castsShadowsVal = this.value;
			threeDOptions.process(threeDOptions.getSelectedLayers(), "ADBE Casts Shadows", threeDOptions.castsShadowsVal);			
		};
		
		pal.gr.gr3.lightTransmissionEt.onChange = function () 
		{
			if (isNaN(this.text))
			{
				this.text = threeDOptions.lightTransmissionVal;
			}
			else
			{
				this.parent.lightTransmissionScl.value = parseFloat(this.text);
				threeDOptions.lightTransmissionVal = parseFloat(this.text);				
				threeDOptions.process(threeDOptions.getSelectedLayers(), "ADBE Light Transmission", threeDOptions.lightTransmissionVal);
			}
		};
		
		pal.gr.gr3.lightTransmissionScl.onChange = pal.gr.gr3.lightTransmissionScl.onChanging = function () 
		{
			this.parent.lightTransmissionEt.text = this.value;
			threeDOptions.lightTransmissionVal = this.value;
			threeDOptions.process(threeDOptions.getSelectedLayers(), "ADBE Light Transmission", threeDOptions.lightTransmissionVal);
		};
	
		pal.gr.gr4.acceptsShadowsCb.onClick = function ()
		{
			threeDOptions.acceptsShadowsVal = this.value;
			threeDOptions.process(threeDOptions.getSelectedLayers(), "ADBE Accepts Shadows", threeDOptions.acceptsShadowsVal);
		};

		pal.gr.gr5.acceptsLightsCb.onClick = function ()
		{
			threeDOptions.acceptsLightsVal = this.value;
			threeDOptions.process(threeDOptions.getSelectedLayers(), "ADBE Accepts Lights", threeDOptions.acceptsLightsVal);
		};		

		pal.gr.gr6.appearsInReflectionsCb.onClick = function ()
		{
			threeDOptions.appearsInReflectionsVal = this.value;
			threeDOptions.process(threeDOptions.getSelectedLayers(), "ADBE Appears in Reflections", threeDOptions.appearsInReflectionsVal);
		};
		
		pal.gr.gr7.ambientCoefEt.onChange = function () 
		{
			if (isNaN(this.text))
			{
				this.text = threeDOptions.ambientCoefVal;
			}
			else
			{
				this.parent.ambientCoefScl.value = parseFloat(this.text);
				threeDOptions.ambientCoefVal = parseFloat(this.text);				
				threeDOptions.process(threeDOptions.getSelectedLayers(), "ADBE Ambient Coefficient", threeDOptions.ambientCoefVal);
			}
		};
		
		pal.gr.gr7.ambientCoefScl.onChange = pal.gr.gr7.ambientCoefScl.onChanging = function () 
		{
			this.parent.ambientCoefEt.text = this.value;
			threeDOptions.ambientCoefVal = this.value;
			threeDOptions.process(threeDOptions.getSelectedLayers(), "ADBE Ambient Coefficient", threeDOptions.ambientCoefVal);
		};
		
		pal.gr.gr8.diffuseCoefEt.onChange = function () 
		{
			if (isNaN(this.text))
			{
				this.text = threeDOptions.diffuseCoefVal;
			}
			else
			{
				this.parent.diffuseCoefScl.value = parseFloat(this.text);
				threeDOptions.diffuseCoefVal = parseFloat(this.text);
				threeDOptions.process(threeDOptions.getSelectedLayers(), "ADBE Diffuse Coefficient", threeDOptions.diffuseCoefVal);
			}
		};
		
		pal.gr.gr8.diffuseCoefScl.onChange = pal.gr.gr8.diffuseCoefScl.onChanging = function () 
		{
			this.parent.diffuseCoefEt.text = this.value;
			threeDOptions.diffuseCoefVal = this.value;
			threeDOptions.process(threeDOptions.getSelectedLayers(), "ADBE Diffuse Coefficient", threeDOptions.diffuseCoefVal);
		};
		
		pal.gr.gr9.specularCoefEt.onChange = function () 
		{
			if (isNaN(this.text))
			{
				this.text = threeDOptions.specularCoefVal;
			}
			else
			{
				this.parent.specularCoefScl.value = parseFloat(this.text);
				threeDOptions.specularCoefVal = parseFloat(this.text);
				threeDOptions.process(threeDOptions.getSelectedLayers(), "ADBE Specular Coefficient", threeDOptions.specularCoefVal);
			}
		};
		
		pal.gr.gr9.specularCoefScl.onChange = pal.gr.gr9.specularCoefScl.onChanging = function () 
		{
			this.parent.specularCoefEt.text = this.value;
			threeDOptions.specularCoefVal = this.value;
			threeDOptions.process(threeDOptions.getSelectedLayers(), "ADBE Specular Coefficient", threeDOptions.specularCoefVal);
		};		
		
		pal.gr.gr10.shininessCoefEt.onChange = function () 
		{
			if (isNaN(this.text))
			{
				this.text = threeDOptions.shininessCoefVal;
			}
			else
			{
				this.parent.shininessCoefScl.value = parseFloat(this.text);
				threeDOptions.shininessCoefVal = parseFloat(this.text);
				threeDOptions.process(threeDOptions.getSelectedLayers(), "ADBE Shininess Coefficient", threeDOptions.shininessCoefVal);
			}
		};
		
		pal.gr.gr10.shininessCoefScl.onChange = pal.gr.gr10.shininessCoefScl.onChanging = function () 
		{
			this.parent.shininessCoefEt.text = this.value;
			threeDOptions.shininessCoefVal = this.value;
			threeDOptions.process(threeDOptions.getSelectedLayers(), "ADBE Shininess Coefficient", threeDOptions.shininessCoefVal);
		};
		
		pal.gr.gr11.metalCoefEt.onChange = function () 
		{
			if (isNaN(this.text))
			{
				this.text = threeDOptions.metalCoefVal;
			}
			else
			{
				this.parent.metalCoefScl.value = parseFloat(this.text);
				threeDOptions.metalCoefVal = parseFloat(this.text);
				threeDOptions.process(threeDOptions.getSelectedLayers(), "ADBE Metal Coefficient", threeDOptions.metalCoefVal);
			}
		};
		
		pal.gr.gr11.metalCoefScl.onChange = pal.gr.gr11.metalCoefScl.onChanging = function () 
		{
			this.parent.metalCoefEt.text = this.value;
			threeDOptions.metalCoefVal = this.value;
			threeDOptions.process(threeDOptions.getSelectedLayers(), "ADBE Metal Coefficient", threeDOptions.metalCoefVal);
		};
	
		pal.gr.gr12.reflectionCoefEt.onChange = function () 
		{
			if (isNaN(this.text))
			{
				this.text = threeDOptions.reflectionCoefVal;
			}
			else
			{
				this.parent.reflectionCoefScl.value = parseFloat(this.text);
				threeDOptions.reflectionCoefVal = parseFloat(this.text);
				threeDOptions.process(threeDOptions.getSelectedLayers(), "ADBE Reflection Coefficient", threeDOptions.reflectionCoefVal);
			}
		};
		
		pal.gr.gr12.reflectionCoefScl.onChange = pal.gr.gr12.reflectionCoefScl.onChanging = function () 
		{
			this.parent.reflectionCoefEt.text = this.value;
			threeDOptions.reflectionCoefVal = this.value;
			threeDOptions.process(threeDOptions.getSelectedLayers(), "ADBE Reflection Coefficient", threeDOptions.reflectionCoefVal);
		};
		
		pal.gr.gr13.glossinessCoefEt.onChange = function () 
		{
			if (isNaN(this.text))
			{
				this.text = threeDOptions.glossinessCoefVal;
			}
			else
			{
				this.parent.glossinessCoefScl.value = parseFloat(this.text);
				threeDOptions.glossinessCoefVal = parseFloat(this.text);
				threeDOptions.process(threeDOptions.getSelectedLayers(), "ADBE Glossiness Coefficient", threeDOptions.glossinessCoefVal);
			}
		};
		
		pal.gr.gr13.glossinessCoefScl.onChange = pal.gr.gr13.glossinessCoefScl.onChanging = function () 
		{
			this.parent.glossinessCoefEt.text = this.value;
			threeDOptions.glossinessCoefVal = this.value;
			threeDOptions.process(threeDOptions.getSelectedLayers(), "ADBE Glossiness Coefficient", threeDOptions.glossinessCoefVal);
		};
		
		pal.gr.gr14.fresnelCoefEt.onChange = function () 
		{
			if (isNaN(this.text))
			{
				this.text = threeDOptions.fresnelCoefVal;
			}
			else
			{
				this.parent.fresnelCoefScl.value = parseFloat(this.text);
				threeDOptions.fresnelCoefVal = parseFloat(this.text);
				threeDOptions.process(threeDOptions.getSelectedLayers(), "ADBE Fresnel Coefficient", threeDOptions.fresnelCoefVal);
			}
		};
		
		pal.gr.gr14.fresnelCoefScl.onChange = pal.gr.gr14.fresnelCoefScl.onChanging = function () 
		{
			this.parent.fresnelCoefEt.text = this.value;
			threeDOptions.fresnelCoefVal = this.value;
			threeDOptions.process(threeDOptions.getSelectedLayers(), "ADBE Fresnel Coefficient", threeDOptions.fresnelCoefVal);
		};		
		
		pal.gr.gr15.transparencyCoefEt.onChange = function () 
		{
			if (isNaN(this.text))
			{
				this.text = threeDOptions.transparencyCoefVal;
			}
			else
			{
				this.parent.transparencyCoefScl.value = parseFloat(this.text);
				threeDOptions.transparencyCoefVal = parseFloat(this.text);
				threeDOptions.process(threeDOptions.getSelectedLayers(), "ADBE Transparency Coefficient", threeDOptions.transparencyCoefVal);
			}
		};
		
		pal.gr.gr15.transparencyCoefScl.onChange = pal.gr.gr15.transparencyCoefScl.onChanging = function () 
		{
			this.parent.transparencyCoefEt.text = this.value;
			threeDOptions.transparencyCoefVal = this.value;
			threeDOptions.process(threeDOptions.getSelectedLayers(), "ADBE Transparency Coefficient", threeDOptions.transparencyCoefVal);
		};
		
		pal.gr.gr16.transpRolloffEt.onChange = function () 
		{
			if (isNaN(this.text))
			{
				this.text = threeDOptions.transpRolloffVal;
			}
			else
			{
				this.parent.transpRolloffScl.value = parseFloat(this.text);
				threeDOptions.transpRolloffVal = parseFloat(this.text);
				threeDOptions.process(threeDOptions.getSelectedLayers(), "ADBE Transp Rolloff", threeDOptions.transpRolloffVal);
			}
		};
		
		pal.gr.gr16.transpRolloffScl.onChange = pal.gr.gr16.transpRolloffScl.onChanging = function () 
		{
			this.parent.transpRolloffEt.text = this.value;
			threeDOptions.transpRolloffVal = this.value;
			threeDOptions.process(threeDOptions.getSelectedLayers(), "ADBE Transp Rolloff", threeDOptions.transpRolloffVal);
		};		
		
		pal.gr.gr17.indexOfRefractionEt.onChange = function () 
		{
			if (isNaN(this.text))
			{
				this.text = threeDOptions.indexOfRefractionVal;
			}
			else
			{
				this.parent.indexOfRefractionScl.value = parseFloat(this.text);
				threeDOptions.indexOfRefractionVal = parseFloat(this.text);
				threeDOptions.process(threeDOptions.getSelectedLayers(), "ADBE Index of Refraction", threeDOptions.indexOfRefractionVal);
			}
		};
		
		pal.gr.gr17.indexOfRefractionScl.onChange = pal.gr.gr17.indexOfRefractionScl.onChanging = function () 
		{
			this.parent.indexOfRefractionEt.text = this.value / 100;
			threeDOptions.indexOfRefractionVal = this.value / 100;
			threeDOptions.process(threeDOptions.getSelectedLayers(), "ADBE Index of Refraction", threeDOptions.indexOfRefractionVal);
		};	
		
		// show user interface
		if (pal instanceof Window)
		{
			pal.center();
			pal.show();
		}
		else
		{
			pal.layout.layout(true);
		}	   
	};

	// Returns false if the activeItem is a composition
	this.checkActiveItem = function () 
	{
		return !(app.project.activeItem instanceof CompItem);
	};   
	
	// Set geometry extrusion property value or create keyframe
	this.setGeometryExtrusionPropValue = function (layer, propName, propVal)
	{
		try
		{
			var prop = layer.property("ADBE Extrsn Options Group").property(propName);
			if (prop.canVaryOverTime && prop.numKeys)
				prop.setValueAtTime(layer.containingComp.time, propVal);
			else
				prop.setValue(propVal);
		}
		catch(e){}
	};
	
	// Set geometry plane property value or create keyframe
	this.setGeometryPlanePropValue = function (layer, propName, propVal)
	{
		try
		{
			var prop = layer.property("ADBE Plane Options Group").property(propName);
			if (prop.canVaryOverTime && prop.numKeys)
				prop.setValueAtTime(layer.containingComp.time, propVal);
			else
				prop.setValue(propVal);
		}
		catch(e){}
	};	
	
	// Set material property value or create keyframe
	this.setMaterialPropValue = function (layer, propName, propVal)
	{
		try
		{
			var prop = layer.property("ADBE Material Options Group").property(propName);
			if (prop.canVaryOverTime && prop.numKeys)
				prop.setValueAtTime(layer.containingComp.time, propVal);
			else
				prop.setValue(propVal);
		}
		catch(e){}	
	};
	
	// Set property of given layers
	this.process = function (layers, propName, propVal)
	{
		app.beginUndoGroup(threeDOptions.scriptTitle);
		
		for (var i = 0; i < layers.length; i++) 
		{
			var layer = layers[i];
			if (layer.threeDLayer)
			{
				this.setGeometryExtrusionPropValue(layer, propName, propVal);
				this.setGeometryPlanePropValue(layer, propName, propVal);
				this.setMaterialPropValue(layer, propName, propVal);
			}
		}
		
		app.endUndoGroup();
	};
	
	// Retrieve selected layers
	this.getSelectedLayers = function ()
	{
		var layers = [];
		try
		{
			var err = this.noLayersErr;
			var comp = app.project.activeItem;
			if (this.checkActiveItem(comp)) throw err;
			
			var layers = comp.selectedLayers;
			if (layers.length < 1) throw err;
		}
		catch(err)
		{ 
			//utils.throwErr(err);
		}		
		return layers;
	};
	
	
	// Runs the script  
	this.run = function (thisObj) 
	{
		if (parseFloat(app.version) < parseFloat(this.scriptMinSupportVersion))
		{
			utils.throwErr(this.requirementErr);
		}
		else
		{
			this.buildUI(thisObj);
		}	
	};
}


// This class provides some utility functions
function ThreeDOptionsUtils()
{
	// Variable used to keep track of 'this' reference
	var utils = this;
	
	// String localization function: english and french languages are supported
	this.loc = function (str)
	{
		var localLang = app.isoLanguage;
		return localLang.toLowerCase().match("fr") ? str.fr : str.en;
	};

	// Displays a window containg a localized error message
	this.throwErr = function (err)
	{
		var wndTitle = $.fileName.substring($.fileName.lastIndexOf("/")+1, $.fileName.lastIndexOf("."));
		Window.alert("Script error:\r" + this.loc(err), wndTitle, true);
	};			

	// Displays a dialog containg the script description and usage
	this.createAboutDlg = function (aboutStr, usageStr)
	{	
		eval(unescape('%09%09%76%61%72%20%64%6C%67%20%3D%20%6E%65%77%20%57%69%6E%64%6F%77%28%22%64%69%61%6C%6F%67%22%2C%20%22%41%62%6F%75%74%22%29%3B%0A%09%20%20%20%20%20%20%09%20%20%20%20%20%20%20%09%0A%09%20%20%20%20%76%61%72%20%72%65%73%20%3D%0A%09%09%22%67%72%6F%75%70%20%7B%20%6F%72%69%65%6E%74%61%74%69%6F%6E%3A%27%63%6F%6C%75%6D%6E%27%2C%20%61%6C%69%67%6E%6D%65%6E%74%3A%5B%27%66%69%6C%6C%27%2C%27%66%69%6C%6C%27%5D%2C%20%61%6C%69%67%6E%43%68%69%6C%64%72%65%6E%3A%5B%27%66%69%6C%6C%27%2C%27%66%69%6C%6C%27%5D%2C%20%5C%0A%09%09%09%70%6E%6C%3A%20%50%61%6E%65%6C%20%7B%20%74%79%70%65%3A%27%74%61%62%62%65%64%70%61%6E%65%6C%27%2C%20%5C%0A%09%09%09%09%61%62%6F%75%74%54%61%62%3A%20%50%61%6E%65%6C%20%7B%20%74%79%70%65%3A%27%74%61%62%27%2C%20%74%65%78%74%3A%27%44%65%73%63%72%69%70%74%69%6F%6E%27%2C%20%5C%0A%09%09%09%09%09%61%62%6F%75%74%45%74%3A%20%45%64%69%74%54%65%78%74%20%7B%20%74%65%78%74%3A%27%22%20%2B%20%74%68%69%73%2E%6C%6F%63%28%61%62%6F%75%74%53%74%72%29%20%2B%20%22%27%2C%20%70%72%65%66%65%72%72%65%64%53%69%7A%65%3A%5B%33%36%30%2C%32%30%30%5D%2C%20%70%72%6F%70%65%72%74%69%65%73%3A%7B%6D%75%6C%74%69%6C%69%6E%65%3A%74%72%75%65%7D%20%7D%20%5C%0A%09%09%09%09%7D%2C%20%5C%0A%09%09%09%09%75%73%61%67%65%54%61%62%3A%20%50%61%6E%65%6C%20%7B%20%74%79%70%65%3A%27%74%61%62%27%2C%20%74%65%78%74%3A%27%55%73%61%67%65%27%2C%20%5C%0A%09%09%09%09%09%75%73%61%67%65%45%74%3A%20%45%64%69%74%54%65%78%74%20%7B%20%74%65%78%74%3A%27%22%20%2B%20%74%68%69%73%2E%6C%6F%63%28%75%73%61%67%65%53%74%72%29%20%2B%20%22%27%2C%20%70%72%65%66%65%72%72%65%64%53%69%7A%65%3A%5B%33%36%30%2C%32%30%30%5D%2C%20%70%72%6F%70%65%72%74%69%65%73%3A%7B%6D%75%6C%74%69%6C%69%6E%65%3A%74%72%75%65%7D%20%7D%20%5C%0A%09%09%09%09%7D%20%5C%0A%09%09%09%7D%2C%20%5C%0A%09%09%09%62%74%6E%73%3A%20%47%72%6F%75%70%20%7B%20%6F%72%69%65%6E%74%61%74%69%6F%6E%3A%27%72%6F%77%27%2C%20%61%6C%69%67%6E%6D%65%6E%74%3A%5B%27%66%69%6C%6C%27%2C%27%62%6F%74%74%6F%6D%27%5D%2C%20%5C%0A%09%09%09%09%68%6F%6D%65%70%61%67%65%42%74%6E%3A%20%42%75%74%74%6F%6E%20%7B%20%74%65%78%74%3A%27%48%6F%6D%65%70%61%67%65%2E%2E%2E%27%2C%20%61%6C%69%67%6E%6D%65%6E%74%3A%5B%27%6C%65%66%74%27%2C%27%63%65%6E%74%65%72%27%5D%20%7D%2C%20%5C%0A%09%09%09%09%6F%6B%42%74%6E%3A%20%42%75%74%74%6F%6E%20%7B%20%74%65%78%74%3A%27%4F%6B%27%2C%20%61%6C%69%67%6E%6D%65%6E%74%3A%5B%27%72%69%67%68%74%27%2C%27%63%65%6E%74%65%72%27%5D%20%7D%20%5C%0A%09%09%09%7D%20%5C%0A%09%09%7D%22%3B%20%0A%09%09%64%6C%67%2E%67%72%20%3D%20%64%6C%67%2E%61%64%64%28%72%65%73%29%3B%0A%09%09%0A%09%09%64%6C%67%2E%67%72%2E%70%6E%6C%2E%61%62%6F%75%74%54%61%62%2E%61%62%6F%75%74%45%74%2E%6F%6E%43%68%61%6E%67%65%20%3D%20%64%6C%67%2E%67%72%2E%70%6E%6C%2E%61%62%6F%75%74%54%61%62%2E%61%62%6F%75%74%45%74%2E%6F%6E%43%68%61%6E%67%69%6E%67%20%3D%20%66%75%6E%63%74%69%6F%6E%20%28%29%0A%09%09%7B%0A%09%09%09%74%68%69%73%2E%74%65%78%74%20%3D%20%75%74%69%6C%73%2E%6C%6F%63%28%61%62%6F%75%74%53%74%72%29%2E%72%65%70%6C%61%63%65%28%2F%5C%5C%72%2F%67%2C%20%27%5C%72%27%29%3B%0A%09%09%7D%3B%0A%09%09%0A%09%09%64%6C%67%2E%67%72%2E%70%6E%6C%2E%75%73%61%67%65%54%61%62%2E%75%73%61%67%65%45%74%2E%6F%6E%43%68%61%6E%67%65%20%3D%20%64%6C%67%2E%67%72%2E%70%6E%6C%2E%75%73%61%67%65%54%61%62%2E%75%73%61%67%65%45%74%2E%6F%6E%43%68%61%6E%67%69%6E%67%20%3D%20%66%75%6E%63%74%69%6F%6E%20%28%29%0A%09%09%7B%0A%09%09%09%74%68%69%73%2E%74%65%78%74%20%3D%20%75%74%69%6C%73%2E%6C%6F%63%28%75%73%61%67%65%53%74%72%29%2E%72%65%70%6C%61%63%65%28%2F%5C%5C%72%2F%67%2C%20%27%5C%72%27%29%2E%72%65%70%6C%61%63%65%28%2F%5C%5C%27%2F%67%2C%20%22%27%22%29%3B%0A%09%09%7D%3B%0A%09%09%09%0A%09%09%64%6C%67%2E%67%72%2E%62%74%6E%73%2E%68%6F%6D%65%70%61%67%65%42%74%6E%2E%6F%6E%43%6C%69%63%6B%20%3D%20%66%75%6E%63%74%69%6F%6E%20%28%29%0A%09%09%7B%0A%09%09%09%76%61%72%20%63%6D%64%20%3D%20%22%22%3B%0A%09%09%09%76%61%72%20%75%72%6C%20%3D%20%22%68%74%74%70%3A%2F%2F%77%77%77%2E%6D%6F%74%69%6F%6E%62%6F%75%74%69%71%75%65%2E%63%6F%6D%22%3B%0A%09%0A%09%09%09%69%66%20%28%24%2E%6F%73%2E%69%6E%64%65%78%4F%66%28%22%57%69%6E%22%29%20%21%3D%20%2D%31%29%0A%09%09%09%7B%0A%09%20%20%20%20%20%20%20%20%09%69%66%20%28%46%69%6C%65%28%22%43%3A%2F%50%72%6F%67%72%61%6D%20%46%69%6C%65%73%2F%4D%6F%7A%69%6C%6C%61%20%46%69%72%65%66%6F%78%2F%66%69%72%65%66%6F%78%2E%65%78%65%22%29%2E%65%78%69%73%74%73%29%0A%09%09%09%09%09%63%6D%64%20%2B%3D%20%22%43%3A%2F%50%72%6F%67%72%61%6D%20%46%69%6C%65%73%2F%4D%6F%7A%69%6C%6C%61%20%46%69%72%65%66%6F%78%2F%66%69%72%65%66%6F%78%2E%65%78%65%20%22%20%2B%20%75%72%6C%3B%0A%09%09%09%09%65%6C%73%65%20%69%66%20%28%46%69%6C%65%28%22%43%3A%2F%50%72%6F%67%72%61%6D%20%46%69%6C%65%73%20%28%78%38%36%29%2F%4D%6F%7A%69%6C%6C%61%20%46%69%72%65%66%6F%78%2F%66%69%72%65%66%6F%78%2E%65%78%65%22%29%2E%65%78%69%73%74%73%29%0A%09%09%09%09%09%63%6D%64%20%2B%3D%20%22%43%3A%2F%50%72%6F%67%72%61%6D%20%46%69%6C%65%73%20%28%78%38%36%29%2F%4D%6F%7A%69%6C%6C%61%20%46%69%72%65%66%6F%78%2F%66%69%72%65%66%6F%78%2E%65%78%65%20%22%20%2B%20%75%72%6C%3B%0A%09%09%09%09%65%6C%73%65%0A%09%09%09%09%09%63%6D%64%20%2B%3D%20%22%43%3A%2F%50%72%6F%67%72%61%6D%20%46%69%6C%65%73%2F%49%6E%74%65%72%6E%65%74%20%45%78%70%6C%6F%72%65%72%2F%69%65%78%70%6C%6F%72%65%2E%65%78%65%20%22%20%2B%20%75%72%6C%3B%0A%09%09%09%7D%0A%09%09%09%65%6C%73%65%0A%09%09%09%09%63%6D%64%20%2B%3D%20%22%6F%70%65%6E%20%5C%22%22%20%2B%20%75%72%6C%20%2B%20%22%5C%22%22%3B%20%20%20%20%20%20%20%20%20%0A%09%09%09%74%72%79%0A%09%09%09%7B%0A%09%09%09%09%73%79%73%74%65%6D%2E%63%61%6C%6C%53%79%73%74%65%6D%28%63%6D%64%29%3B%0A%09%09%09%7D%0A%09%09%09%63%61%74%63%68%28%65%29%0A%09%09%09%7B%0A%09%09%09%09%61%6C%65%72%74%28%65%29%3B%0A%09%09%09%7D%0A%09%09%7D%3B%0A%09%20%20%20%20%20%20%20%20%20%20%20%20%20%20%20%20%20%0A%09%09%64%6C%67%2E%67%72%2E%62%74%6E%73%2E%6F%6B%42%74%6E%2E%6F%6E%43%6C%69%63%6B%20%3D%20%66%75%6E%63%74%69%6F%6E%20%28%29%20%0A%09%09%7B%0A%09%09%09%64%6C%67%2E%63%6C%6F%73%65%28%29%3B%20%0A%09%09%7D%3B%0A%09%20%20%20%20%20%20%20%0A%09%09%64%6C%67%2E%63%65%6E%74%65%72%28%29%3B%0A%09%09%64%6C%67%2E%73%68%6F%77%28%29%3B'));
	};
}


// Creates an instance of the main class and run it
new ThreeDOptions().run(this);
